@@ -1,4 +1,7 @@
 # flake8: noqa
+from importlib import metadata
+__version__ = metadata.version(__name__)
+
 from monaco.mc_case import *
 from monaco.mc_sim import *
 from monaco.mc_var import *
@@ -10,6 +13,8 @@ from monaco.order_statistics import *
 from monaco.integration_statistics import *
 from monaco.dvars_sensitivity import *
 from monaco.helper_functions import *
+from monaco.case_runners import *
 from monaco.mc_sampling import *
 from monaco.mc_plot import *
 from monaco.mc_multi_plot import *
+from monaco.tqdm_dask_distributed import *
