@@ -3,16 +3,18 @@ from __future__ import annotations
 
 import os
 import numpy as np
-import dill
+import dask
+from dask.distributed import Client
+import cloudpickle
 import pathlib
 from datetime import datetime, timedelta
 from monaco.mc_case import Case
 from monaco.mc_var import InVar, OutVar
 from monaco.mc_enums import SimFunctions, SampleMethod
-from monaco.helper_functions import (get_list, slice_by_index, vprint, vwarn,
-                                     vwrite, hash_str_repeatable)
-from psutil import cpu_count
-from pathos.pools import ThreadPool as Pool
+from monaco.helper_functions import get_list, vprint, vwarn, hash_str_repeatable
+from monaco.case_runners import preprocess_case, run_case, postprocess_case
+from monaco.tqdm_dask_distributed import tqdm_dask
+from monaco.dvars_sensitivity import calc_sensitivities
 from tqdm import tqdm
 from typing import Callable, Any, Iterable
 from scipy.stats import rv_continuous, rv_discrete
@@ -39,9 +41,11 @@ class Sim:
         The random sampling method to use.
     seed : int, default: np.random.get_state(legacy=False)['state']['key'][0]
         The random number to seed the simulation.
-    cores : int, default: psutil.cpu_count(logical=False)
-        The number of cores to use for running the simulation. Defaults to the
-        number of physical cores on the machine.
+    singlethreaded : bool, default: False
+        Whether to run single threaded rather than using dask.
+    daskkwargs : dict, default: dict()
+        Kwargs to pass to the dask Client constructor, see:
+        https://distributed.dask.org/en/stable/api.html#client
     verbose : bool, default: True
         Whether to print out warning and status messages.
     debug : bool, default: False
@@ -100,12 +104,6 @@ class Sim:
         The covariance matrix between all of the scalar variables.
     covvarlist : list[str]
         The names of all the scalar variables.
-    pbar0 : tqdm.tqdm
-        Handle for the preprocessing progress bar.
-    pbar1 : tqdm.tqdm
-        Handle for the running progress bar.
-    pbar2 : tqdm.tqdm
-        Handle for the postprocessing progress bar.
     runsimid : int
         The unique ID for a particular run of this simulation.
     ncases : int
@@ -118,13 +116,14 @@ class Sim:
                  firstcaseismedian : bool = False,
                  samplemethod      : SampleMethod = SampleMethod.SOBOL_RANDOM,
                  seed              : int  = np.random.get_state(legacy=False)['state']['key'][0],
-                 cores             : int  = cpu_count(logical=False),
+                 singlethreaded    : bool = False,
+                 daskkwargs        : dict = dict(),
                  verbose           : bool = True,
                  debug             : bool = False,
                  savesimdata       : bool = True,
                  savecasedata      : bool = True,
                  resultsdir        : str | pathlib.Path = None,
-                 ):
+                 ) -> None:
 
         self.checkFcnsInput(fcns)
 
@@ -136,7 +135,8 @@ class Sim:
         self.firstcaseismedian = firstcaseismedian
         self.samplemethod = samplemethod
         self.seed = seed
-        self.cores = cores
+        self.singlethreaded = singlethreaded
+        self.daskkwargs = daskkwargs
         self.savesimdata = savesimdata
         self.savecasedata = savecasedata
 
@@ -174,29 +174,41 @@ class Sim:
         self.covs       : np.ndarray = None
         self.covvarlist : list[str] = None
 
-        self.pbar0 : tqdm = None
-        self.pbar1 : tqdm = None
-        self.pbar2 : tqdm = None
-
         self.runsimid : int = None
 
         self.ncases : int = ndraws + 1
         self.setFirstCaseMedian(firstcaseismedian)
         self.setNDraws(self.ndraws)  # will regen runsimid
 
+        self.client = None
+        self.cluster = None
+        if not self.singlethreaded:
+            self.initDaskClient()
 
-    def __getstate__(self):
+
+    def __del__(self) -> None:
+        if self.client is not None:
+            self.client.close()
+
+
+    def __getstate__(self) -> dict:
         """Function for pickling self to save to file."""
         state = self.__dict__.copy()
+        state['client'] = None  # don't save cluster to file
+        state['cluster'] = None  # don't save cluster to file
         state['cases'] = []  # don't save case data when pickling self
         return state
 
 
-    def __setstate__(self, state):
+    def __setstate__(self,
+                     state: dict,
+                     ) -> None:
         """Function to unpickle self when loading from file."""
         self.__dict__.update(state)
         if self.savecasedata:
             self.loadCases()
+        if not self.singlethreaded:
+            self.initDaskClient()
 
 
     def checkFcnsInput(self,
@@ -213,11 +225,11 @@ class Sim:
         """
         if set(fcns.keys()) \
            != {SimFunctions.PREPROCESS, SimFunctions.RUN, SimFunctions.POSTPROCESS}:
-            raise ValueError(f'Sim argument fcns={fcns} must have keys ' +
+            raise ValueError(f'Sim argument {fcns=} must have keys ' +
                              f'{SimFunctions.PREPROCESS}, {SimFunctions.RUN}, ' +
                              f'and {SimFunctions.POSTPROCESS}')
         if any(not callable(f) for f in fcns.values()):
-            raise ValueError(f'Sim argument fcns={fcns} must contain functions as values')
+            raise ValueError(f'Sim argument {fcns=} must contain functions as values')
 
 
     def setFirstCaseMedian(self,
@@ -242,6 +254,23 @@ class Sim:
                 invar.setFirstCaseMedian(firstcaseismedian)
 
 
+    def initDaskClient(self):
+        """
+        Initialize the dask distributed client.
+        """
+        if not self.singlethreaded:
+            self.client = Client(**self.daskkwargs)
+            self.cluster = self.client.cluster
+
+            nworkers = len(self.cluster.workers)
+            nthreads = nworkers * self.cluster.worker_spec[0]['options']['nthreads']
+            memory = nworkers * self.cluster.worker_spec[0]['options']['memory_limit']
+            vprint(self.verbose,
+                   f'Dask cluster initiated with {nworkers} workers,' +
+                   f'{nthreads} threads, {memory/2**30:0.2f} GiB memory.')
+            vprint(self.verbose, f'Dask dashboard link: {self.cluster.dashboard_link}')
+
+
     def addInVar(self,
                  name       : str,
                  dist       : rv_discrete | rv_continuous,
@@ -414,10 +443,11 @@ class Sim:
 
         vprint(self.verbose, f'Simulation complete! Runtime: {self.runtime}', flush=True)
 
+        if self.savecasedata:
+            self.saveCasesToFile()
+
         if self.savesimdata:
-            vprint(self.verbose, 'Saving sim results to file...', flush=True)
             self.saveSimToFile()
-            vprint(self.verbose, f"Sim results saved in '{self.filepath}'", flush=True)
 
 
     def genRunSimID(self) -> None:
@@ -487,65 +517,53 @@ class Sim:
             preprocessed.
         """
         cases_downselect = self.downselectCases(cases=cases)
+        preprocessedcases = []
 
-        if self.verbose:
-            self.pbar0 = tqdm(total=len(cases_downselect), desc='Preprocessing cases',
-                              unit=' cases', position=0)
-
-        if self.cores == 1:
-            for ncase in cases_downselect:
-                self.preProcessCase(case=self.cases[ncase])
+        # Single-threaded for loop
+        if self.singlethreaded:
+            if self.verbose:
+                pbar = tqdm(total=len(cases_downselect), desc='Preprocessing cases',
+                            unit=' cases', position=0)
+            for case in self.cases:
+                if case.ncase in cases_downselect:
+                    case.haspreprocessed = False
+                    case = preprocess_case(self.fcns[SimFunctions.PREPROCESS],
+                                            case, self.debug, self.verbose)
+                    preprocessedcases.append(case)
+                    if self.verbose:
+                        pbar.update(1)
+            if self.verbose:
+                pbar.refresh()
+                pbar.close()
 
+        # Dask parallel processing
         else:
-            p = Pool(self.cores)
             try:
-                cases_downselect = set(slice_by_index(self.cases, cases_downselect))
-                cases = p.imap(self.preProcessCase, cases_downselect)
-                cases = list(cases)
-                p.terminate()
-                p.restart()
+                for case in self.cases:
+                    if case.ncase in cases_downselect:
+                        case.haspreprocessed = False
+                        case_delayed = dask.delayed(preprocess_case)(
+                            self.fcns[SimFunctions.PREPROCESS], case,
+                            self.debug, self.verbose)
+                        preprocessedcases.append(case_delayed)
+
+                if self.verbose:
+                    x = dask.persist(preprocessedcases)
+                    tqdm_dask(x, total=len(cases_downselect),
+                              desc='Preprocessing cases',
+                              unit=' cases', position=0)
+                    preprocessedcases = dask.compute(*x)[0]
+                else:
+                    preprocessedcases = dask.compute(*preprocessedcases)
+
             except KeyboardInterrupt:
-                p.terminate()
-                p.restart()
                 raise
 
-        if self.verbose:
-            self.pbar0.refresh()
-            self.pbar0.close()
-            self.pbar0 = None
-
-
-    def preProcessCase(self,
-                       case : Case,
-                       ) -> Case:
-        """
-        Preprocess a single Monte Carlo case.
-
-        Parameters
-        ----------
-        case : monaco.mc_case.Case
-            The case to preprocess.
-
-        Returns
-        -------
-        case : monaco.mc_case.Case
-            The same case, preprocessed.
-        """
-        try:
-            case.siminput = self.fcns[SimFunctions.PREPROCESS](case)
-            self.casespreprocessed.add(case.ncase)
-            case.haspreprocessed = True
-
-        except Exception:
-            if self.debug:
-                raise
-            else:
-                vwarn(self.verbose, f'\nPreprocessing case {case.ncase} failed')
-
-        if self.pbar0 is not None:
-            self.pbar0.update(1)
-
-        return case
+        # Save out results
+        for case in preprocessedcases:
+            if case.haspreprocessed:
+                self.cases[case.ncase] = case
+                self.casespreprocessed.add(case.ncase)
 
 
     def runCases(self,
@@ -564,85 +582,56 @@ class Sim:
             this simulation run is generated.
         """
         cases_downselect = self.downselectCases(cases=cases)
+        runcases = []
 
         if not calledfromrunsim:
             self.runsimid = self.genID()
 
-        if self.verbose:
-            self.pbar1 = tqdm(total=len(cases_downselect), desc='Running cases',
-                              unit=' cases', position=0)
-
-        if self.cores == 1:
-            for ncase in cases_downselect:
-                self.runCase(case=self.cases[ncase])
+        # Single-threaded for loop
+        if self.singlethreaded:
+            if self.verbose:
+                pbar = tqdm(total=len(cases_downselect), desc='Running cases',
+                            unit=' cases', position=0)
+            for case in self.cases:
+                if case.ncase in cases_downselect:
+                    case.hasrun = False
+                    case = run_case(self.fcns[SimFunctions.RUN], case,
+                                    self.debug, self.verbose, self.runsimid)
+                    runcases.append(case)
+                    if self.verbose:
+                        pbar.update(1)
+            if self.verbose:
+                pbar.refresh()
+                pbar.close()
 
+        # Dask parallel processing
         else:
-            p = Pool(self.cores)
             try:
-                cases_downselect = set(slice_by_index(self.cases, cases_downselect))
-                casesrun = p.imap(self.runCase, cases_downselect)
-                # below is a dummy function to ensure we wait for imap to finish
-                casesrun = list(casesrun)
-                p.terminate()
-                p.restart()
+                for case in self.cases:
+                    if case.ncase in cases_downselect:
+                        case.hasrun = False
+                        case_delayed = dask.delayed(run_case)(
+                            self.fcns[SimFunctions.RUN], case,
+                            self.debug, self.verbose, self.runsimid)
+                        runcases.append(case_delayed)
+
+                if self.verbose:
+                    x = dask.persist(runcases)
+                    tqdm_dask(x, total=len(cases_downselect),
+                              desc='Running cases',
+                              unit=' cases', position=0)
+                    runcases = dask.compute(*x)[0]
+                else:
+                    runcases = dask.compute(*runcases)
+
             except KeyboardInterrupt:
-                p.terminate()
-                p.restart()
                 raise
 
-        if self.verbose:
-            self.pbar1.refresh()
-            self.pbar1.close()
-            self.pbar1 = None
-
-        if self.savecasedata:
-            vprint(self.verbose, f"\nRaw case results saved in '{self.resultsdir}'",
-                   end='', flush=True)
-
-
-    def runCase(self,
-                case : Case,
-                ) -> None:
-        """
-        Run a single Monte Carlo case.
-
-        Parameters
-        ----------
-        case : monaco.mc_case.Case
-            The case to run.
-
-        Returns
-        -------
-        case : monaco.mc_case.Case
-            The same case, ran.
-        """
-        try:
-            case.starttime = datetime.now()
-            case.simrawoutput = self.fcns[SimFunctions.RUN](*get_list(case.siminput))
-            case.endtime = datetime.now()
-            case.runtime = case.endtime - case.starttime
-            case.runsimid = self.runsimid
-            case.hasrun = True
-
-            if self.savecasedata:
-                filepath = self.resultsdir / f'{self.name}_{case.ncase}.mccase'
-                case.filepath = filepath
-                try:
-                    filepath.unlink()
-                except FileNotFoundError:
-                    pass
-                with open(filepath, 'wb') as file:
-                    dill.dump(case, file, protocol=dill.HIGHEST_PROTOCOL)
-
-            self.casesrun.add(case.ncase)
-
-        except Exception:
-            if self.debug:
-                raise
-            vwrite(self.verbose, f'\nRunning case {case.ncase} failed')
-
-        if self.pbar1 is not None:
-            self.pbar1.update(1)
+        # Save out results
+        for case in runcases:
+            if case.hasrun:
+                self.cases[case.ncase] = case
+                self.casesrun.add(case.ncase)
 
 
     def postProcessCases(self,
@@ -658,64 +647,53 @@ class Sim:
             postprocessed.
         """
         cases_downselect = self.downselectCases(cases=cases)
+        postprocessedcases = []
 
-        if self.verbose:
-            self.pbar2 = tqdm(total=len(cases_downselect), desc='Postprocessing cases',
-                              unit=' cases', position=0)
-
-        if self.cores == 1:
-            for ncase in cases_downselect:
-                self.postProcessCase(case=self.cases[ncase])
+        # Single-threaded for loop
+        if self.singlethreaded:
+            if self.verbose:
+                pbar = tqdm(total=len(cases_downselect), desc='Preprocessing cases',
+                            unit=' cases', position=0)
+            for case in self.cases:
+                if case.ncase in cases_downselect:
+                    case.haspostprocessed = False
+                    case = postprocess_case(self.fcns[SimFunctions.POSTPROCESS],
+                                            case, self.debug, self.verbose)
+                    postprocessedcases.append(case)
+                    if self.verbose:
+                        pbar.update(1)
+            if self.verbose:
+                pbar.refresh()
+                pbar.close()
 
+        # Dask parallel processing
         else:
-            p = Pool(self.cores)
             try:
-                cases_downselect = set(slice_by_index(self.cases, cases_downselect))
-                casespostprocessed = p.imap(self.postProcessCase, cases_downselect)
-                # below is a dummy function to ensure we wait for imap to finish
-                casespostprocessed = list(casespostprocessed)
-                p.terminate()
-                p.restart()
+                for case in self.cases:
+                    if case.ncase in cases_downselect:
+                        case.haspostprocessed = False
+                        case_delayed = dask.delayed(postprocess_case)(
+                            self.fcns[SimFunctions.POSTPROCESS], case,
+                            self.debug, self.verbose)
+                        postprocessedcases.append(case_delayed)
+
+                if self.verbose:
+                    x = dask.persist(postprocessedcases)
+                    tqdm_dask(x, total=len(cases_downselect),
+                              desc='Postprocessing cases',
+                              unit=' cases', position=0)
+                    postprocessedcases = dask.compute(*x)[0]
+                else:
+                    postprocessedcases = dask.compute(*postprocessedcases)
+
             except KeyboardInterrupt:
-                p.terminate()
-                p.restart()
                 raise
 
-        if self.verbose:
-            self.pbar2.refresh()
-            self.pbar2.close()
-            self.pbar2 = None
-
-
-    def postProcessCase(self,
-                        case : Case,
-                        ) -> None:
-        """
-        Postprocess a single Monte Carlo case.
-
-        Parameters
-        ----------
-        case : monaco.mc_case.Case
-            The case to postprocess.
-
-        Returns
-        -------
-        case : monaco.mc_case.Case
-            The same case, postprocessed.
-        """
-        try:
-            self.fcns[SimFunctions.POSTPROCESS](case, *get_list(case.simrawoutput))
-            self.casespostprocessed.add(case.ncase)
-            case.haspostprocessed = True
-
-        except Exception:
-            if self.debug:
-                raise
-            else:
-                vwrite(self.verbose, f'\nPostprocessing case {case.ncase} failed')
-
-        if self.pbar2 is not None:
-            self.pbar2.update(1)
+        # Save out results
+        for case in postprocessedcases:
+            if case.haspostprocessed:
+                self.cases[case.ncase] = case
+                self.casespostprocessed.add(case.ncase)
 
 
     def genOutVars(self) -> None:
@@ -766,6 +744,40 @@ class Sim:
         return scalaroutvars
 
 
+    def calcSensitivities(self,
+                          outvarnames: None | str | Iterable[str] = None,
+                          ) -> None:
+        """
+        Calculate the sensitivity indices for the specified outvars.
+
+        Parameters
+        ----------
+        outvarnames : None | str | Iterable[str] (default: None)
+            The outvar names to calculate sensitivity indices for. If None,
+            then calculates sensitivities for all scalar outvars.
+        """
+        if outvarnames is None:
+            outvarname = list(self.scalarOutVars.keys())
+        outvarnames = get_list(outvarnames)
+
+        for outvarname in outvarnames:
+            if not self.outvars[outvarname].isscalar:
+                vwarn(self.verbose, f"Output variable '{outvarname}' is not scalar," +
+                                     'skipping sensitivity calculations.')
+            else:
+                vprint(self.verbose, f"Calculating sensitivity indices for '{outvarname}'.")
+                sensitivities, ratios = calc_sensitivities(self, outvarname)
+
+                sensitivities_dict = dict()
+                ratios_dict = dict()
+                for i, name in enumerate(self.invars.keys()):
+                    sensitivities_dict[name] = sensitivities[i]
+                    ratios_dict[name] = ratios[i]
+
+                self.outvars[outvarname].sensitivity_indices = sensitivities_dict
+                self.outvars[outvarname].sensitivity_ratios = ratios_dict
+
+
     def genCovarianceMatrix(self) -> None:
         """
         Generate the covariance matrix and correlation coefficients between all
@@ -887,13 +899,47 @@ class Sim:
     def saveSimToFile(self) -> None:
         """Save the simulation to a .mcsim file"""
         if self.savesimdata:
+            vprint(self.verbose, 'Saving sim results to file...', flush=True)
+
             try:
                 self.filepath.unlink()
             except FileNotFoundError:
                 pass
             self.filepath.touch()
             with open(self.filepath, 'wb') as file:
-                dill.dump(self, file, protocol=dill.HIGHEST_PROTOCOL)
+                cloudpickle.dump(self, file)
+
+            vprint(self.verbose, f"Sim results saved in '{self.filepath}'", flush=True)
+
+
+    def saveCasesToFile(self,
+                        cases : None | int | Iterable[int] = None,
+                        ) -> None:
+        """
+        Save the specified cases to .mccase files.
+
+        Parameters
+        ----------
+        cases : None | int | Iterable[int]
+            The cases to save. If None, save all cases.
+        """
+        cases_downselect = self.downselectCases(cases=cases)
+
+        if self.savecasedata:
+            vprint(self.verbose, 'Saving cases to file...', flush=True)
+
+            for ncase in cases_downselect:
+                filepath = self.resultsdir / f'{self.name}_{ncase}.mccase'
+                self.cases[ncase].filepath = filepath
+                try:
+                    filepath.unlink()
+                except FileNotFoundError:
+                    pass
+                with open(filepath, 'wb') as file:
+                    cloudpickle.dump(self.cases[ncase], file)
+
+            vprint(self.verbose, f"Raw case results saved in '{self.resultsdir}'",
+                   flush=True)
 
 
     def loadCases(self) -> None:
@@ -907,14 +953,14 @@ class Sim:
         casesnotloaded        = self.allCases()
         casesnotpostprocessed = self.allCases()
 
-        pbar = tqdm(total=len(self.casesrun), unit=' cases', desc='Loading', position=0)
+        # pbar = tqdm(total=len(self.casesrun), unit=' cases', desc='Loading', position=0)
 
         for ncase in self.casesrun:
             filepath = self.resultsdir / f'{self.name}_{ncase}.mccase'
             try:
                 with open(filepath, 'rb') as file:
                     try:
-                        case = dill.load(file)
+                        case = cloudpickle.load(file)
                         if (not case.haspreprocessed) \
                             or (not case.hasrun) \
                             or (case.runtime is None):  # only load case if it completed running
@@ -932,17 +978,19 @@ class Sim:
                             casesnotloaded.remove(ncase)
                             if ncase in self.casespostprocessed:
                                 casesnotpostprocessed.remove(ncase)
+
                     except Exception:
                         vwarn(self.verbose, f'Unknown error loading {filepath.name}')
+
             except FileNotFoundError:
                 vwarn(self.verbose, f'{filepath.name} expected but not found')
-            pbar.update(1)
+            # pbar.update(1)
 
         self.casespreprocessed  = set(casesloaded)
         self.casesrun           = set(casesloaded)
         self.casespostprocessed = set(casesloaded) - casesnotpostprocessed
-        pbar.refresh()
-        pbar.close()
+        # pbar.refresh()
+        # pbar.close()
         vprint(self.verbose, f'\nData for {len(casesloaded)}/{self.ncases} cases loaded from disk',
                flush=True)
 
